# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.currency_and_amount import CurrencyAndAmount
from lusid.models.otc_confirmation import OtcConfirmation
from lusid.models.perpetual_property import PerpetualProperty
from lusid.models.resource_id import ResourceId
from lusid.models.strategy import Strategy
from lusid.models.transaction_price import TransactionPrice

class TransactionRequest(BaseModel):
    """
    TransactionRequest
    """
    transaction_id:  StrictStr = Field(...,alias="transactionId", description="The unique identifier of the transaction.") 
    type:  StrictStr = Field(...,alias="type", description="The type of the transaction, for example 'Buy' or 'Sell'. The transaction type must have been pre-configured using the System Configuration API. If not, this operation will succeed but you are not able to calculate holdings for the portfolio that include this transaction.") 
    instrument_identifiers: Dict[str, Optional[StrictStr]] = Field(description="A set of instrument identifiers that can resolve the transaction to a unique instrument.", alias="instrumentIdentifiers")
    transaction_date:  StrictStr = Field(...,alias="transactionDate", description="The date of the transaction.") 
    settlement_date:  StrictStr = Field(...,alias="settlementDate", description="The settlement date of the transaction.") 
    units: Union[StrictFloat, StrictInt] = Field(description="The number of units of the transacted instrument.")
    transaction_price: Optional[TransactionPrice] = Field(default=None, alias="transactionPrice")
    total_consideration: CurrencyAndAmount = Field(alias="totalConsideration")
    exchange_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The exchange rate between the transaction and settlement currency (settlement currency being represented by TotalConsideration.Currency). For example, if the transaction currency is USD and the settlement currency is GBP, this would be the appropriate USD/GBP rate.", alias="exchangeRate")
    transaction_currency:  Optional[StrictStr] = Field(None,alias="transactionCurrency", description="The transaction currency.") 
    properties: Optional[Dict[str, PerpetualProperty]] = Field(default=None, description="A list of unique transaction properties and associated values to store for the transaction. Each property must be from the 'Transaction' domain.")
    counterparty_id:  Optional[StrictStr] = Field(None,alias="counterpartyId", description="The identifier for the counterparty of the transaction.") 
    source:  Optional[StrictStr] = Field(None,alias="source", description="The source of the transaction. This is used to look up the appropriate transaction group set in the transaction type configuration.") 
    otc_confirmation: Optional[OtcConfirmation] = Field(default=None, alias="otcConfirmation")
    order_id: Optional[ResourceId] = Field(default=None, alias="orderId")
    allocation_id: Optional[ResourceId] = Field(default=None, alias="allocationId")
    custodian_account_id: Optional[ResourceId] = Field(default=None, alias="custodianAccountId")
    transaction_group_id:  Optional[StrictStr] = Field(None,alias="transactionGroupId", description="The identifier for grouping economic events across multiple transactions") 
    strategy_tag: Optional[List[Strategy]] = Field(default=None, description="A list of strategies representing the allocation of units across multiple sub-holding keys", alias="strategyTag")
    __properties = ["transactionId", "type", "instrumentIdentifiers", "transactionDate", "settlementDate", "units", "transactionPrice", "totalConsideration", "exchangeRate", "transactionCurrency", "properties", "counterpartyId", "source", "otcConfirmation", "orderId", "allocationId", "custodianAccountId", "transactionGroupId", "strategyTag"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransactionRequest:
        """Create an instance of TransactionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of transaction_price
        if self.transaction_price:
            _dict['transactionPrice'] = self.transaction_price.to_dict()
        # override the default output from pydantic by calling `to_dict()` of total_consideration
        if self.total_consideration:
            _dict['totalConsideration'] = self.total_consideration.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in properties (dict)
        _field_dict = {}
        if self.properties:
            for _key in self.properties:
                if self.properties[_key]:
                    _field_dict[_key] = self.properties[_key].to_dict()
            _dict['properties'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of otc_confirmation
        if self.otc_confirmation:
            _dict['otcConfirmation'] = self.otc_confirmation.to_dict()
        # override the default output from pydantic by calling `to_dict()` of order_id
        if self.order_id:
            _dict['orderId'] = self.order_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of allocation_id
        if self.allocation_id:
            _dict['allocationId'] = self.allocation_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of custodian_account_id
        if self.custodian_account_id:
            _dict['custodianAccountId'] = self.custodian_account_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in strategy_tag (list)
        _items = []
        if self.strategy_tag:
            for _item in self.strategy_tag:
                if _item:
                    _items.append(_item.to_dict())
            _dict['strategyTag'] = _items
        # set to None if exchange_rate (nullable) is None
        # and __fields_set__ contains the field
        if self.exchange_rate is None and "exchange_rate" in self.__fields_set__:
            _dict['exchangeRate'] = None

        # set to None if transaction_currency (nullable) is None
        # and __fields_set__ contains the field
        if self.transaction_currency is None and "transaction_currency" in self.__fields_set__:
            _dict['transactionCurrency'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        # set to None if counterparty_id (nullable) is None
        # and __fields_set__ contains the field
        if self.counterparty_id is None and "counterparty_id" in self.__fields_set__:
            _dict['counterpartyId'] = None

        # set to None if source (nullable) is None
        # and __fields_set__ contains the field
        if self.source is None and "source" in self.__fields_set__:
            _dict['source'] = None

        # set to None if transaction_group_id (nullable) is None
        # and __fields_set__ contains the field
        if self.transaction_group_id is None and "transaction_group_id" in self.__fields_set__:
            _dict['transactionGroupId'] = None

        # set to None if strategy_tag (nullable) is None
        # and __fields_set__ contains the field
        if self.strategy_tag is None and "strategy_tag" in self.__fields_set__:
            _dict['strategyTag'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransactionRequest:
        """Create an instance of TransactionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TransactionRequest.parse_obj(obj)

        _obj = TransactionRequest.parse_obj({
            "transaction_id": obj.get("transactionId"),
            "type": obj.get("type"),
            "instrument_identifiers": obj.get("instrumentIdentifiers"),
            "transaction_date": obj.get("transactionDate"),
            "settlement_date": obj.get("settlementDate"),
            "units": obj.get("units"),
            "transaction_price": TransactionPrice.from_dict(obj.get("transactionPrice")) if obj.get("transactionPrice") is not None else None,
            "total_consideration": CurrencyAndAmount.from_dict(obj.get("totalConsideration")) if obj.get("totalConsideration") is not None else None,
            "exchange_rate": obj.get("exchangeRate"),
            "transaction_currency": obj.get("transactionCurrency"),
            "properties": dict(
                (_k, PerpetualProperty.from_dict(_v))
                for _k, _v in obj.get("properties").items()
            )
            if obj.get("properties") is not None
            else None,
            "counterparty_id": obj.get("counterpartyId"),
            "source": obj.get("source"),
            "otc_confirmation": OtcConfirmation.from_dict(obj.get("otcConfirmation")) if obj.get("otcConfirmation") is not None else None,
            "order_id": ResourceId.from_dict(obj.get("orderId")) if obj.get("orderId") is not None else None,
            "allocation_id": ResourceId.from_dict(obj.get("allocationId")) if obj.get("allocationId") is not None else None,
            "custodian_account_id": ResourceId.from_dict(obj.get("custodianAccountId")) if obj.get("custodianAccountId") is not None else None,
            "transaction_group_id": obj.get("transactionGroupId"),
            "strategy_tag": [Strategy.from_dict(_item) for _item in obj.get("strategyTag")] if obj.get("strategyTag") is not None else None
        })
        return _obj

TransactionRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class AccountingMethod(str, Enum):
    """
    AccountingMethod
    """

    """
    allowed enum values
    """
    DEFAULT = 'Default'
    AVERAGECOST = 'AverageCost'
    FIRSTINFIRSTOUT = 'FirstInFirstOut'
    LASTINFIRSTOUT = 'LastInFirstOut'
    HIGHESTCOSTFIRST = 'HighestCostFirst'
    LOWESTCOSTFIRST = 'LowestCostFirst'
    PRORATEBYUNITS = 'ProRateByUnits'
    PRORATEBYCOST = 'ProRateByCost'
    PRORATEBYCOSTPORTFOLIOCURRENCY = 'ProRateByCostPortfolioCurrency'
    INTRADAYTHENFIRSTINFIRSTOUT = 'IntraDayThenFirstInFirstOut'
    LONGTERMHIGHESTCOSTFIRST = 'LongTermHighestCostFirst'
    LONGTERMHIGHESTCOSTFIRSTPORTFOLIOCURRENCY = 'LongTermHighestCostFirstPortfolioCurrency'
    HIGHESTCOSTFIRSTPORTFOLIOCURRENCY = 'HighestCostFirstPortfolioCurrency'
    LOWESTCOSTFIRSTPORTFOLIOCURRENCY = 'LowestCostFirstPortfolioCurrency'
    MAXIMUMLOSSMINIMUMGAIN = 'MaximumLossMinimumGain'
    MAXIMUMLOSSMINIMUMGAINPORTFOLIOCURRENCY = 'MaximumLossMinimumGainPortfolioCurrency'

    @classmethod
    def from_json(cls, json_str: str) -> AccountingMethod:
        """Create an instance of AccountingMethod from a JSON string"""
        return AccountingMethod(json.loads(json_str))



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.data_model_membership import DataModelMembership
from lusid.models.link import Link
from lusid.models.lusid_instrument import LusidInstrument
from lusid.models.model_property import ModelProperty
from lusid.models.relationship import Relationship
from lusid.models.resource_id import ResourceId
from lusid.models.settlement_cycle import SettlementCycle
from lusid.models.staged_modifications_info import StagedModificationsInfo
from lusid.models.version import Version

class Instrument(BaseModel):
    """
    A list of instruments.  # noqa: E501
    """
    href:  Optional[StrictStr] = Field(None,alias="href", description="The specific Uniform Resource Identifier (URI) for this resource at the requested effective and asAt datetime.") 
    scope:  Optional[StrictStr] = Field(None,alias="scope", description="The scope in which the instrument lies.") 
    lusid_instrument_id:  StrictStr = Field(...,alias="lusidInstrumentId", description="The unique LUSID Instrument Identifier (LUID) of the instrument.") 
    version: Version
    staged_modifications: Optional[StagedModificationsInfo] = Field(default=None, alias="stagedModifications")
    name:  StrictStr = Field(...,alias="name", description="The name of the instrument.") 
    identifiers: Dict[str, Optional[StrictStr]] = Field(description="The set of identifiers that can be used to identify the instrument.")
    properties: Optional[List[ModelProperty]] = Field(default=None, description="The requested instrument properties. These will be from the 'Instrument' domain.")
    lookthrough_portfolio: Optional[ResourceId] = Field(default=None, alias="lookthroughPortfolio")
    instrument_definition: Optional[LusidInstrument] = Field(default=None, alias="instrumentDefinition")
    state:  StrictStr = Field(...,alias="state", description="The state of of the instrument at the asAt datetime of this version of the instrument definition. The available values are: Active, Inactive, Deleted") 
    asset_class:  Optional[StrictStr] = Field(None,alias="assetClass", description="The nominal asset class of the instrument, e.g. InterestRates, FX, Inflation, Equities, Credit, Commodities, etc. The available values are: InterestRates, FX, Inflation, Equities, Credit, Commodities, Money, Unknown") 
    dom_ccy:  Optional[StrictStr] = Field(None,alias="domCcy", description="The domestic currency, meaning the currency in which the instrument would typically be expected to pay cashflows, e.g. a share in AAPL being USD.") 
    relationships: Optional[List[Relationship]] = Field(default=None, description="A set of relationships associated to the instrument.")
    settlement_cycle: Optional[SettlementCycle] = Field(default=None, alias="settlementCycle")
    data_model_membership: Optional[DataModelMembership] = Field(default=None, alias="dataModelMembership")
    links: Optional[List[Link]] = None
    __properties = ["href", "scope", "lusidInstrumentId", "version", "stagedModifications", "name", "identifiers", "properties", "lookthroughPortfolio", "instrumentDefinition", "state", "assetClass", "domCcy", "relationships", "settlementCycle", "dataModelMembership", "links"]

    @validator('state')
    def state_validate_enum(cls, value):
        """Validates the enum"""

        # Finbourne have removed enum validation on all models, except for this use case:
        # Workflow and notification application SDK use the property name 'type' as the discriminator on a number of classes.
        # During instantiation, the value of 'type' is checked against the enum values, 
        

        # check it's a class that uses the 'type' property as a discriminator
        # list of classes can be found by searching for 'actual_instance: Union[' in the generated code
        if 'Instrument' not in [ 
                                    # For notification application classes
                                    'AmazonSqsNotificationType',
                                    'AmazonSqsNotificationTypeResponse',
                                    'AmazonSqsPrincipalAuthNotificationType',
                                    'AmazonSqsPrincipalAuthNotificationTypeResponse',
                                    'AzureServiceBusTypeResponse',
                                    'AzureServiceBusNotificationType',
                                    'EmailNotificationType',
                                    'EmailNotificationTypeResponse',
                                    'SmsNotificationType',
                                    'SmsNotificationTypeResponse',
                                    'WebhookNotificationType',
                                    'WebhookNotificationTypeResponse',
                        
                                    # For workflow application classes
                                    'CreateChildTasksAction', 
                                    'RunWorkerAction', 
                                    'TriggerParentTaskAction',
                                    'CreateChildTasksActionResponse', 
                                    'RunWorkerActionResponse',
                                    'TriggerParentTaskActionResponse',
                                    'CreateNewTaskActivity',
                                    'UpdateMatchingTasksActivity',
                                    'CreateNewTaskActivityResponse', 
                                    'UpdateMatchingTasksActivityResponse',
                                    'Fail', 
                                    'GroupReconciliation', 
                                    'HealthCheck', 
                                    'LuminesceView', 
                                    'SchedulerJob', 
                                    'Sleep',
                                    'FailResponse', 
                                    'GroupReconciliationResponse', 
                                    'HealthCheckResponse', 
                                    'LuminesceViewResponse', 
                                    'SchedulerJobResponse', 
                                    'SleepResponse',
                                    'Library',
                                    'LibraryResponse',
                                    'DayRegularity',
                                    'RelativeMonthRegularity',
                                    'SpecificMonthRegularity',
                                    'WeekRegularity',
                                    'YearRegularity']:
           return value
        
        # Only validate the 'type' property of the class
        if "state" != "type":
            return value

        if value not in ['Active', 'Inactive', 'Deleted']:
            raise ValueError("must be one of enum values ('Active', 'Inactive', 'Deleted')")
        return value

    @validator('asset_class')
    def asset_class_validate_enum(cls, value):
        """Validates the enum"""

        # Finbourne have removed enum validation on all models, except for this use case:
        # Workflow and notification application SDK use the property name 'type' as the discriminator on a number of classes.
        # During instantiation, the value of 'type' is checked against the enum values, 
        

        # check it's a class that uses the 'type' property as a discriminator
        # list of classes can be found by searching for 'actual_instance: Union[' in the generated code
        if 'Instrument' not in [ 
                                    # For notification application classes
                                    'AmazonSqsNotificationType',
                                    'AmazonSqsNotificationTypeResponse',
                                    'AmazonSqsPrincipalAuthNotificationType',
                                    'AmazonSqsPrincipalAuthNotificationTypeResponse',
                                    'AzureServiceBusTypeResponse',
                                    'AzureServiceBusNotificationType',
                                    'EmailNotificationType',
                                    'EmailNotificationTypeResponse',
                                    'SmsNotificationType',
                                    'SmsNotificationTypeResponse',
                                    'WebhookNotificationType',
                                    'WebhookNotificationTypeResponse',
                        
                                    # For workflow application classes
                                    'CreateChildTasksAction', 
                                    'RunWorkerAction', 
                                    'TriggerParentTaskAction',
                                    'CreateChildTasksActionResponse', 
                                    'RunWorkerActionResponse',
                                    'TriggerParentTaskActionResponse',
                                    'CreateNewTaskActivity',
                                    'UpdateMatchingTasksActivity',
                                    'CreateNewTaskActivityResponse', 
                                    'UpdateMatchingTasksActivityResponse',
                                    'Fail', 
                                    'GroupReconciliation', 
                                    'HealthCheck', 
                                    'LuminesceView', 
                                    'SchedulerJob', 
                                    'Sleep',
                                    'FailResponse', 
                                    'GroupReconciliationResponse', 
                                    'HealthCheckResponse', 
                                    'LuminesceViewResponse', 
                                    'SchedulerJobResponse', 
                                    'SleepResponse',
                                    'Library',
                                    'LibraryResponse',
                                    'DayRegularity',
                                    'RelativeMonthRegularity',
                                    'SpecificMonthRegularity',
                                    'WeekRegularity',
                                    'YearRegularity']:
           return value
        
        # Only validate the 'type' property of the class
        if "asset_class" != "type":
            return value

        if value is None:
            return value

        if value not in ['InterestRates', 'FX', 'Inflation', 'Equities', 'Credit', 'Commodities', 'Money', 'Unknown']:
            raise ValueError("must be one of enum values ('InterestRates', 'FX', 'Inflation', 'Equities', 'Credit', 'Commodities', 'Money', 'Unknown')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Instrument:
        """Create an instance of Instrument from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of staged_modifications
        if self.staged_modifications:
            _dict['stagedModifications'] = self.staged_modifications.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in properties (list)
        _items = []
        if self.properties:
            for _item in self.properties:
                if _item:
                    _items.append(_item.to_dict())
            _dict['properties'] = _items
        # override the default output from pydantic by calling `to_dict()` of lookthrough_portfolio
        if self.lookthrough_portfolio:
            _dict['lookthroughPortfolio'] = self.lookthrough_portfolio.to_dict()
        # override the default output from pydantic by calling `to_dict()` of instrument_definition
        if self.instrument_definition:
            _dict['instrumentDefinition'] = self.instrument_definition.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in relationships (list)
        _items = []
        if self.relationships:
            for _item in self.relationships:
                if _item:
                    _items.append(_item.to_dict())
            _dict['relationships'] = _items
        # override the default output from pydantic by calling `to_dict()` of settlement_cycle
        if self.settlement_cycle:
            _dict['settlementCycle'] = self.settlement_cycle.to_dict()
        # override the default output from pydantic by calling `to_dict()` of data_model_membership
        if self.data_model_membership:
            _dict['dataModelMembership'] = self.data_model_membership.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if scope (nullable) is None
        # and __fields_set__ contains the field
        if self.scope is None and "scope" in self.__fields_set__:
            _dict['scope'] = None

        # set to None if properties (nullable) is None
        # and __fields_set__ contains the field
        if self.properties is None and "properties" in self.__fields_set__:
            _dict['properties'] = None

        # set to None if dom_ccy (nullable) is None
        # and __fields_set__ contains the field
        if self.dom_ccy is None and "dom_ccy" in self.__fields_set__:
            _dict['domCcy'] = None

        # set to None if relationships (nullable) is None
        # and __fields_set__ contains the field
        if self.relationships is None and "relationships" in self.__fields_set__:
            _dict['relationships'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Instrument:
        """Create an instance of Instrument from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Instrument.parse_obj(obj)

        _obj = Instrument.parse_obj({
            "href": obj.get("href"),
            "scope": obj.get("scope"),
            "lusid_instrument_id": obj.get("lusidInstrumentId"),
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "staged_modifications": StagedModificationsInfo.from_dict(obj.get("stagedModifications")) if obj.get("stagedModifications") is not None else None,
            "name": obj.get("name"),
            "identifiers": obj.get("identifiers"),
            "properties": [ModelProperty.from_dict(_item) for _item in obj.get("properties")] if obj.get("properties") is not None else None,
            "lookthrough_portfolio": ResourceId.from_dict(obj.get("lookthroughPortfolio")) if obj.get("lookthroughPortfolio") is not None else None,
            "instrument_definition": LusidInstrument.from_dict(obj.get("instrumentDefinition")) if obj.get("instrumentDefinition") is not None else None,
            "state": obj.get("state"),
            "asset_class": obj.get("assetClass"),
            "dom_ccy": obj.get("domCcy"),
            "relationships": [Relationship.from_dict(_item) for _item in obj.get("relationships")] if obj.get("relationships") is not None else None,
            "settlement_cycle": SettlementCycle.from_dict(obj.get("settlementCycle")) if obj.get("settlementCycle") is not None else None,
            "data_model_membership": DataModelMembership.from_dict(obj.get("dataModelMembership")) if obj.get("dataModelMembership") is not None else None,
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

Instrument.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.break_code_source import BreakCodeSource
from lusid.models.group_reconciliation_definition_comparison_ruleset_ids import GroupReconciliationDefinitionComparisonRulesetIds
from lusid.models.group_reconciliation_definition_currencies import GroupReconciliationDefinitionCurrencies
from lusid.models.group_reconciliation_definition_portfolio_entity_ids import GroupReconciliationDefinitionPortfolioEntityIds
from lusid.models.group_reconciliation_definition_recipe_ids import GroupReconciliationDefinitionRecipeIds
from lusid.models.primary_schedule import PrimarySchedule
from lusid.models.resource_id import ResourceId
from lusid.models.transaction_date_windows import TransactionDateWindows

class CreateGroupReconciliationDefinitionRequest(BaseModel):
    """
    CreateGroupReconciliationDefinitionRequest
    """
    id: Optional[ResourceId] = None
    display_name:  StrictStr = Field(...,alias="displayName", description="The name of the Group Reconciliation Definition") 
    description:  Optional[StrictStr] = Field(None,alias="description", description="The description of the Group Reconciliation Definition") 
    portfolio_entity_ids: GroupReconciliationDefinitionPortfolioEntityIds = Field(alias="portfolioEntityIds")
    recipe_ids: Optional[GroupReconciliationDefinitionRecipeIds] = Field(default=None, alias="recipeIds")
    currencies: Optional[GroupReconciliationDefinitionCurrencies] = None
    transaction_date_windows: Optional[TransactionDateWindows] = Field(default=None, alias="transactionDateWindows")
    comparison_ruleset_ids: Optional[GroupReconciliationDefinitionComparisonRulesetIds] = Field(default=None, alias="comparisonRulesetIds")
    break_code_source: Optional[BreakCodeSource] = Field(default=None, alias="breakCodeSource")
    primary_schedule: Optional[PrimarySchedule] = Field(default=None, alias="primarySchedule")
    __properties = ["id", "displayName", "description", "portfolioEntityIds", "recipeIds", "currencies", "transactionDateWindows", "comparisonRulesetIds", "breakCodeSource", "primarySchedule"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateGroupReconciliationDefinitionRequest:
        """Create an instance of CreateGroupReconciliationDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of id
        if self.id:
            _dict['id'] = self.id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of portfolio_entity_ids
        if self.portfolio_entity_ids:
            _dict['portfolioEntityIds'] = self.portfolio_entity_ids.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recipe_ids
        if self.recipe_ids:
            _dict['recipeIds'] = self.recipe_ids.to_dict()
        # override the default output from pydantic by calling `to_dict()` of currencies
        if self.currencies:
            _dict['currencies'] = self.currencies.to_dict()
        # override the default output from pydantic by calling `to_dict()` of transaction_date_windows
        if self.transaction_date_windows:
            _dict['transactionDateWindows'] = self.transaction_date_windows.to_dict()
        # override the default output from pydantic by calling `to_dict()` of comparison_ruleset_ids
        if self.comparison_ruleset_ids:
            _dict['comparisonRulesetIds'] = self.comparison_ruleset_ids.to_dict()
        # override the default output from pydantic by calling `to_dict()` of break_code_source
        if self.break_code_source:
            _dict['breakCodeSource'] = self.break_code_source.to_dict()
        # override the default output from pydantic by calling `to_dict()` of primary_schedule
        if self.primary_schedule:
            _dict['primarySchedule'] = self.primary_schedule.to_dict()
        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> CreateGroupReconciliationDefinitionRequest:
        """Create an instance of CreateGroupReconciliationDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return CreateGroupReconciliationDefinitionRequest.parse_obj(obj)

        _obj = CreateGroupReconciliationDefinitionRequest.parse_obj({
            "id": ResourceId.from_dict(obj.get("id")) if obj.get("id") is not None else None,
            "display_name": obj.get("displayName"),
            "description": obj.get("description"),
            "portfolio_entity_ids": GroupReconciliationDefinitionPortfolioEntityIds.from_dict(obj.get("portfolioEntityIds")) if obj.get("portfolioEntityIds") is not None else None,
            "recipe_ids": GroupReconciliationDefinitionRecipeIds.from_dict(obj.get("recipeIds")) if obj.get("recipeIds") is not None else None,
            "currencies": GroupReconciliationDefinitionCurrencies.from_dict(obj.get("currencies")) if obj.get("currencies") is not None else None,
            "transaction_date_windows": TransactionDateWindows.from_dict(obj.get("transactionDateWindows")) if obj.get("transactionDateWindows") is not None else None,
            "comparison_ruleset_ids": GroupReconciliationDefinitionComparisonRulesetIds.from_dict(obj.get("comparisonRulesetIds")) if obj.get("comparisonRulesetIds") is not None else None,
            "break_code_source": BreakCodeSource.from_dict(obj.get("breakCodeSource")) if obj.get("breakCodeSource") is not None else None,
            "primary_schedule": PrimarySchedule.from_dict(obj.get("primarySchedule")) if obj.get("primarySchedule") is not None else None
        })
        return _obj

CreateGroupReconciliationDefinitionRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class RollInterestUpdates(BaseModel):
    """
    Describes changes to the interest of a FlexibleDeposit instrument as the result of a DepositRollEvent.  Both the interest to be withdrawn and the interest to be reinvested must be specified (either as an amount or as a percentage).  # noqa: E501
    """
    withdraw_interest_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of interest that should be withdrawn from a FlexibleDeposit as the result of a roll event.", alias="withdrawInterestAmount")
    withdraw_interest_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The percentage of interest that should be withdrawn from a FlexibleDeposit instrument as the result of a roll event.", alias="withdrawInterestPercentage")
    reinvest_interest_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of interest that should be reinvested in a FlexibleDeposit instrument as the result of a roll event.", alias="reinvestInterestAmount")
    reinvest_interest_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The percentage of interest that should be reinvested in a FlexibleDeposit instrument as the result of a roll event.", alias="reinvestInterestPercentage")
    __properties = ["withdrawInterestAmount", "withdrawInterestPercentage", "reinvestInterestAmount", "reinvestInterestPercentage"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RollInterestUpdates:
        """Create an instance of RollInterestUpdates from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if withdraw_interest_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.withdraw_interest_amount is None and "withdraw_interest_amount" in self.__fields_set__:
            _dict['withdrawInterestAmount'] = None

        # set to None if withdraw_interest_percentage (nullable) is None
        # and __fields_set__ contains the field
        if self.withdraw_interest_percentage is None and "withdraw_interest_percentage" in self.__fields_set__:
            _dict['withdrawInterestPercentage'] = None

        # set to None if reinvest_interest_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.reinvest_interest_amount is None and "reinvest_interest_amount" in self.__fields_set__:
            _dict['reinvestInterestAmount'] = None

        # set to None if reinvest_interest_percentage (nullable) is None
        # and __fields_set__ contains the field
        if self.reinvest_interest_percentage is None and "reinvest_interest_percentage" in self.__fields_set__:
            _dict['reinvestInterestPercentage'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RollInterestUpdates:
        """Create an instance of RollInterestUpdates from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RollInterestUpdates.parse_obj(obj)

        _obj = RollInterestUpdates.parse_obj({
            "withdraw_interest_amount": obj.get("withdrawInterestAmount"),
            "withdraw_interest_percentage": obj.get("withdrawInterestPercentage"),
            "reinvest_interest_amount": obj.get("reinvestInterestAmount"),
            "reinvest_interest_percentage": obj.get("reinvestInterestPercentage")
        })
        return _obj

RollInterestUpdates.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.grouped_result_of_address_key import GroupedResultOfAddressKey

class VirtualDocumentRow(BaseModel):
    """
    Rows identified by the composite id, based on the data maps  # noqa: E501
    """
    row_id: Optional[Dict[str, Optional[StrictStr]]] = Field(default=None, description="The identifier for the row. This is keyed by address keys, and values obtained through applying the data map to the documents.", alias="rowId")
    row_data: Optional[GroupedResultOfAddressKey] = Field(default=None, alias="rowData")
    __properties = ["rowId", "rowData"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VirtualDocumentRow:
        """Create an instance of VirtualDocumentRow from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of row_data
        if self.row_data:
            _dict['rowData'] = self.row_data.to_dict()
        # set to None if row_id (nullable) is None
        # and __fields_set__ contains the field
        if self.row_id is None and "row_id" in self.__fields_set__:
            _dict['rowId'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> VirtualDocumentRow:
        """Create an instance of VirtualDocumentRow from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return VirtualDocumentRow.parse_obj(obj)

        _obj = VirtualDocumentRow.parse_obj({
            "row_id": obj.get("rowId"),
            "row_data": GroupedResultOfAddressKey.from_dict(obj.get("rowData")) if obj.get("rowData") is not None else None
        })
        return _obj

VirtualDocumentRow.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class ComplianceParameterType(str, Enum):
    """
    ComplianceParameterType
    """

    """
    allowed enum values
    """
    BOOLCOMPLIANCEPARAMETER = 'BoolComplianceParameter'
    STRINGCOMPLIANCEPARAMETER = 'StringComplianceParameter'
    DECIMALCOMPLIANCEPARAMETER = 'DecimalComplianceParameter'
    DATETIMECOMPLIANCEPARAMETER = 'DateTimeComplianceParameter'
    PROPERTYKEYCOMPLIANCEPARAMETER = 'PropertyKeyComplianceParameter'
    ADDRESSKEYCOMPLIANCEPARAMETER = 'AddressKeyComplianceParameter'
    PORTFOLIOIDCOMPLIANCEPARAMETER = 'PortfolioIdComplianceParameter'
    PORTFOLIOGROUPIDCOMPLIANCEPARAMETER = 'PortfolioGroupIdComplianceParameter'
    STRINGLISTCOMPLIANCEPARAMETER = 'StringListComplianceParameter'
    BOOLLISTCOMPLIANCEPARAMETER = 'BoolListComplianceParameter'
    DATETIMELISTCOMPLIANCEPARAMETER = 'DateTimeListComplianceParameter'
    DECIMALLISTCOMPLIANCEPARAMETER = 'DecimalListComplianceParameter'
    PROPERTYKEYLISTCOMPLIANCEPARAMETER = 'PropertyKeyListComplianceParameter'
    ADDRESSKEYLISTCOMPLIANCEPARAMETER = 'AddressKeyListComplianceParameter'
    PORTFOLIOIDLISTCOMPLIANCEPARAMETER = 'PortfolioIdListComplianceParameter'
    PORTFOLIOGROUPIDLISTCOMPLIANCEPARAMETER = 'PortfolioGroupIdListComplianceParameter'
    INSTRUMENTLISTCOMPLIANCEPARAMETER = 'InstrumentListComplianceParameter'
    FILTERPREDICATECOMPLIANCEPARAMETER = 'FilterPredicateComplianceParameter'
    GROUPFILTERPREDICATECOMPLIANCEPARAMETER = 'GroupFilterPredicateComplianceParameter'
    GROUPBYSELECTORCOMPLIANCEPARAMETER = 'GroupBySelectorComplianceParameter'
    PROPERTYLISTCOMPLIANCEPARAMETER = 'PropertyListComplianceParameter'
    GROUPCALCULATIONCOMPLIANCEPARAMETER = 'GroupCalculationComplianceParameter'

    @classmethod
    def from_json(cls, json_str: str) -> ComplianceParameterType:
        """Create an instance of ComplianceParameterType from a JSON string"""
        return ComplianceParameterType(json.loads(json_str))



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.error_detail import ErrorDetail
from lusid.models.link import Link
from lusid.models.result_data_schema import ResultDataSchema

class BucketedCashFlowResponse(BaseModel):
    """
    BucketedCashFlowResponse
    """
    href:  Optional[StrictStr] = Field(None,alias="href") 
    data: Optional[List[Dict[str, Any]]] = Field(default=None, description="List of dictionary bucketed cash flow result set.  Each dictionary represent a bucketed cashflow result set keyed by AddressKeys.  e.g. dictionary[\"Valuation/CashFlowAmount\"] for the aggregated cash flow amount for the bucket.  e.g. suppose \"RoundUp\" method, then dictionary[\"Valuation/CashFlowDate/RoundUp\"] returns the bucketed cashflow date.")
    report_currency:  Optional[StrictStr] = Field(None,alias="reportCurrency", description="Three letter ISO currency string indicating what currency to report in for ReportCcy denominated queries.  If not present then the currency of the relevant portfolio will be used in its place where relevant.") 
    data_schema: Optional[ResultDataSchema] = Field(default=None, alias="dataSchema")
    failed: Optional[Dict[str, ErrorDetail]] = Field(default=None, description="Information about where instruments have failed to return cashflows in so far as it is available.  e.g., failure to retrieve a market quote for a floating rate instrument.")
    links: Optional[List[Link]] = None
    __properties = ["href", "data", "reportCurrency", "dataSchema", "failed", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BucketedCashFlowResponse:
        """Create an instance of BucketedCashFlowResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of data_schema
        if self.data_schema:
            _dict['dataSchema'] = self.data_schema.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each value in failed (dict)
        _field_dict = {}
        if self.failed:
            for _key in self.failed:
                if self.failed[_key]:
                    _field_dict[_key] = self.failed[_key].to_dict()
            _dict['failed'] = _field_dict
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if href (nullable) is None
        # and __fields_set__ contains the field
        if self.href is None and "href" in self.__fields_set__:
            _dict['href'] = None

        # set to None if data (nullable) is None
        # and __fields_set__ contains the field
        if self.data is None and "data" in self.__fields_set__:
            _dict['data'] = None

        # set to None if report_currency (nullable) is None
        # and __fields_set__ contains the field
        if self.report_currency is None and "report_currency" in self.__fields_set__:
            _dict['reportCurrency'] = None

        # set to None if failed (nullable) is None
        # and __fields_set__ contains the field
        if self.failed is None and "failed" in self.__fields_set__:
            _dict['failed'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BucketedCashFlowResponse:
        """Create an instance of BucketedCashFlowResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BucketedCashFlowResponse.parse_obj(obj)

        _obj = BucketedCashFlowResponse.parse_obj({
            "href": obj.get("href"),
            "data": obj.get("data"),
            "report_currency": obj.get("reportCurrency"),
            "data_schema": ResultDataSchema.from_dict(obj.get("dataSchema")) if obj.get("dataSchema") is not None else None,
            "failed": dict(
                (_k, ErrorDetail.from_dict(_v))
                for _k, _v in obj.get("failed").items()
            )
            if obj.get("failed") is not None
            else None,
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

BucketedCashFlowResponse.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class LusidEntityResult(BaseModel):
    """
    Represents LUSID entity details for a data quality check result  # noqa: E501
    """
    as_at: Optional[datetime] = Field(default=None, description="The as-at timestamp for the entity", alias="asAt")
    effective_at: Optional[datetime] = Field(default=None, description="The effective-at timestamp for the entity", alias="effectiveAt")
    entity_type:  Optional[StrictStr] = Field(None,alias="entityType", description="The type of the LUSID entity") 
    scope:  Optional[StrictStr] = Field(None,alias="scope", description="The scope of the entity") 
    identifier_key:  Optional[StrictStr] = Field(None,alias="identifierKey", description="The identifier key for the entity") 
    identifier_value:  Optional[StrictStr] = Field(None,alias="identifierValue", description="The identifier value for the entity") 
    entity_unique_id:  Optional[StrictStr] = Field(None,alias="entityUniqueId", description="The unique identifier for the entity") 
    display_name:  Optional[StrictStr] = Field(None,alias="displayName", description="The display name of the entity") 
    __properties = ["asAt", "effectiveAt", "entityType", "scope", "identifierKey", "identifierValue", "entityUniqueId", "displayName"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> LusidEntityResult:
        """Create an instance of LusidEntityResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if entity_type (nullable) is None
        # and __fields_set__ contains the field
        if self.entity_type is None and "entity_type" in self.__fields_set__:
            _dict['entityType'] = None

        # set to None if scope (nullable) is None
        # and __fields_set__ contains the field
        if self.scope is None and "scope" in self.__fields_set__:
            _dict['scope'] = None

        # set to None if identifier_key (nullable) is None
        # and __fields_set__ contains the field
        if self.identifier_key is None and "identifier_key" in self.__fields_set__:
            _dict['identifierKey'] = None

        # set to None if identifier_value (nullable) is None
        # and __fields_set__ contains the field
        if self.identifier_value is None and "identifier_value" in self.__fields_set__:
            _dict['identifierValue'] = None

        # set to None if entity_unique_id (nullable) is None
        # and __fields_set__ contains the field
        if self.entity_unique_id is None and "entity_unique_id" in self.__fields_set__:
            _dict['entityUniqueId'] = None

        # set to None if display_name (nullable) is None
        # and __fields_set__ contains the field
        if self.display_name is None and "display_name" in self.__fields_set__:
            _dict['displayName'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> LusidEntityResult:
        """Create an instance of LusidEntityResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return LusidEntityResult.parse_obj(obj)

        _obj = LusidEntityResult.parse_obj({
            "as_at": obj.get("asAt"),
            "effective_at": obj.get("effectiveAt"),
            "entity_type": obj.get("entityType"),
            "scope": obj.get("scope"),
            "identifier_key": obj.get("identifierKey"),
            "identifier_value": obj.get("identifierValue"),
            "entity_unique_id": obj.get("entityUniqueId"),
            "display_name": obj.get("displayName")
        })
        return _obj

LusidEntityResult.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.append_market_data import AppendMarketData
from lusid.models.complex_market_data_id import ComplexMarketDataId

class AppendComplexMarketDataRequest(BaseModel):
    """
    The details of the point to be appended to a complex market data item.  # noqa: E501
    """
    market_data_id: ComplexMarketDataId = Field(alias="marketDataId")
    append_market_data: AppendMarketData = Field(alias="appendMarketData")
    __properties = ["marketDataId", "appendMarketData"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AppendComplexMarketDataRequest:
        """Create an instance of AppendComplexMarketDataRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of market_data_id
        if self.market_data_id:
            _dict['marketDataId'] = self.market_data_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of append_market_data
        if self.append_market_data:
            _dict['appendMarketData'] = self.append_market_data.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> AppendComplexMarketDataRequest:
        """Create an instance of AppendComplexMarketDataRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return AppendComplexMarketDataRequest.parse_obj(obj)

        _obj = AppendComplexMarketDataRequest.parse_obj({
            "market_data_id": ComplexMarketDataId.from_dict(obj.get("marketDataId")) if obj.get("marketDataId") is not None else None,
            "append_market_data": AppendMarketData.from_dict(obj.get("appendMarketData")) if obj.get("appendMarketData") is not None else None
        })
        return _obj

AppendComplexMarketDataRequest.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.category_settlement_status import CategorySettlementStatus

class TransactionSettlementSummary(BaseModel):
    """
    TransactionSettlementSummary
    """
    overall_status: CategorySettlementStatus = Field(alias="overallStatus")
    stock_status: CategorySettlementStatus = Field(alias="stockStatus")
    cash_status: CategorySettlementStatus = Field(alias="cashStatus")
    deferred_cash_receipt_status: CategorySettlementStatus = Field(alias="deferredCashReceiptStatus")
    __properties = ["overallStatus", "stockStatus", "cashStatus", "deferredCashReceiptStatus"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TransactionSettlementSummary:
        """Create an instance of TransactionSettlementSummary from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of overall_status
        if self.overall_status:
            _dict['overallStatus'] = self.overall_status.to_dict()
        # override the default output from pydantic by calling `to_dict()` of stock_status
        if self.stock_status:
            _dict['stockStatus'] = self.stock_status.to_dict()
        # override the default output from pydantic by calling `to_dict()` of cash_status
        if self.cash_status:
            _dict['cashStatus'] = self.cash_status.to_dict()
        # override the default output from pydantic by calling `to_dict()` of deferred_cash_receipt_status
        if self.deferred_cash_receipt_status:
            _dict['deferredCashReceiptStatus'] = self.deferred_cash_receipt_status.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> TransactionSettlementSummary:
        """Create an instance of TransactionSettlementSummary from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return TransactionSettlementSummary.parse_obj(obj)

        _obj = TransactionSettlementSummary.parse_obj({
            "overall_status": CategorySettlementStatus.from_dict(obj.get("overallStatus")) if obj.get("overallStatus") is not None else None,
            "stock_status": CategorySettlementStatus.from_dict(obj.get("stockStatus")) if obj.get("stockStatus") is not None else None,
            "cash_status": CategorySettlementStatus.from_dict(obj.get("cashStatus")) if obj.get("cashStatus") is not None else None,
            "deferred_cash_receipt_status": CategorySettlementStatus.from_dict(obj.get("deferredCashReceiptStatus")) if obj.get("deferredCashReceiptStatus") is not None else None
        })
        return _obj

TransactionSettlementSummary.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg

from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime


class VendorLibrary(str, Enum):
    """
    VendorLibrary
    """

    """
    allowed enum values
    """
    LUSID = 'Lusid'
    REFINITIVQPS = 'RefinitivQps'
    REFINITIVTRACSWEB = 'RefinitivTracsWeb'
    VOLMASTER = 'VolMaster'
    ISDACDS = 'IsdaCds'
    YIELDBOOK = 'YieldBook'
    LUSIDCALC = 'LusidCalc'

    @classmethod
    def from_json(cls, json_str: str) -> VendorLibrary:
        """Create an instance of VendorLibrary from a JSON string"""
        return VendorLibrary(json.loads(json_str))



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.link import Link
from lusid.models.resource_id import ResourceId
from lusid.models.version import Version

class RelationshipDefinition(BaseModel):
    """
    RelationshipDefinition
    """
    version: Optional[Version] = None
    relationship_definition_id: ResourceId = Field(alias="relationshipDefinitionId")
    source_entity_type:  StrictStr = Field(...,alias="sourceEntityType", description="The entity type of the source entity object.") 
    target_entity_type:  StrictStr = Field(...,alias="targetEntityType", description="The entity type of the target entity object.") 
    display_name:  StrictStr = Field(...,alias="displayName", description="The display name of the relationship.") 
    outward_description:  StrictStr = Field(...,alias="outwardDescription", description="The description to relate source entity object and target entity object") 
    inward_description:  StrictStr = Field(...,alias="inwardDescription", description="The description to relate target entity object and source entity object") 
    life_time:  StrictStr = Field(...,alias="lifeTime", description="Describes how the relationships can change over time.") 
    relationship_cardinality:  StrictStr = Field(...,alias="relationshipCardinality", description="Describes the cardinality of the relationship between source entity and target entity.") 
    links: Optional[List[Link]] = None
    __properties = ["version", "relationshipDefinitionId", "sourceEntityType", "targetEntityType", "displayName", "outwardDescription", "inwardDescription", "lifeTime", "relationshipCardinality", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> RelationshipDefinition:
        """Create an instance of RelationshipDefinition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of version
        if self.version:
            _dict['version'] = self.version.to_dict()
        # override the default output from pydantic by calling `to_dict()` of relationship_definition_id
        if self.relationship_definition_id:
            _dict['relationshipDefinitionId'] = self.relationship_definition_id.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> RelationshipDefinition:
        """Create an instance of RelationshipDefinition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return RelationshipDefinition.parse_obj(obj)

        _obj = RelationshipDefinition.parse_obj({
            "version": Version.from_dict(obj.get("version")) if obj.get("version") is not None else None,
            "relationship_definition_id": ResourceId.from_dict(obj.get("relationshipDefinitionId")) if obj.get("relationshipDefinitionId") is not None else None,
            "source_entity_type": obj.get("sourceEntityType"),
            "target_entity_type": obj.get("targetEntityType"),
            "display_name": obj.get("displayName"),
            "outward_description": obj.get("outwardDescription"),
            "inward_description": obj.get("inwardDescription"),
            "life_time": obj.get("lifeTime"),
            "relationship_cardinality": obj.get("relationshipCardinality"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj

RelationshipDefinition.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime
from lusid.models.lusid_instrument import LusidInstrument
from lusid.models.rounding_convention import RoundingConvention
from lusid.models.schedule import Schedule
from lusid.models.time_zone_conventions import TimeZoneConventions
from lusid.models.trading_conventions import TradingConventions

class ComplexBond(LusidInstrument):
    """
    LUSID representation of a Complex Bond.  Including Floating, Fixed-to-float, Sinkable, Callable, Puttable, and Mortgage Backed Securities.  # noqa: E501
    """
    identifiers: Optional[Dict[str, Optional[StrictStr]]] = Field(default=None, description="External market codes and identifiers for the bond, e.g. ISIN.")
    calculation_type:  Optional[StrictStr] = Field(None,alias="calculationType", description="The calculation type applied to the bond coupon amount. This is required for bonds that have a particular type of computing the period coupon, such as simple compounding,  irregular coupons etc.  The default CalculationType is `Standard`, which returns a coupon amount equal to Principal * Coupon Rate / Coupon Frequency. Coupon Frequency is 12M / Payment Frequency.  Payment Frequency can be 1M, 3M, 6M, 12M etc. So Coupon Frequency can be 12, 4, 2, 1 respectively.    Supported string (enumeration) values are: [Standard, DayCountCoupon, NoCalculationFloater, BrazilFixedCoupon, StandardWithCappedAccruedInterest].") 
    schedules: Optional[List[Schedule]] = Field(default=None, description="schedules.")
    original_issue_price: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The price the complex bond was issued at. This is to be entered as a percentage of par, for example a value of 98.5 would represent 98.5%.", alias="originalIssuePrice")
    rounding_conventions: Optional[List[RoundingConvention]] = Field(default=None, description="Rounding conventions for analytics, if any.", alias="roundingConventions")
    asset_backed: Optional[StrictBool] = Field(default=None, description="If this flag is set to true, then the outstanding notional and principal repayments will be calculated based  on pool factors in the quote store. Usually AssetBacked bonds also require a RollConvention setting of   within the FlowConventions any given rates schedule (to ensure payment dates always happen on the same day  of the month) and US Agency MBSs with Pay Delay features also require their rates schedules to include an  ExDividendConfiguration to drive the lag between interest accrual and payment.", alias="assetBacked")
    asset_pool_identifier:  Optional[StrictStr] = Field(None,alias="assetPoolIdentifier", description="Identifier used to retrieve pool factor information about this bond from the quote store. This is typically  the bond's ISIN, but can also be ClientInternal. Please ensure you align the MarketDataKeyRule with the  correct Quote (Quote.ClientInternal.* or Quote.Isin.*)") 
    trading_conventions: Optional[TradingConventions] = Field(default=None, alias="tradingConventions")
    time_zone_conventions: Optional[TimeZoneConventions] = Field(default=None, alias="timeZoneConventions")
    instrument_type:  StrictStr = Field(...,alias="instrumentType", description="The available values are: QuotedSecurity, InterestRateSwap, FxForward, Future, ExoticInstrument, FxOption, CreditDefaultSwap, InterestRateSwaption, Bond, EquityOption, FixedLeg, FloatingLeg, BespokeCashFlowsLeg, Unknown, TermDeposit, ContractForDifference, EquitySwap, CashPerpetual, CapFloor, CashSettled, CdsIndex, Basket, FundingLeg, FxSwap, ForwardRateAgreement, SimpleInstrument, Repo, Equity, ExchangeTradedOption, ReferenceInstrument, ComplexBond, InflationLinkedBond, InflationSwap, SimpleCashFlowLoan, TotalReturnSwap, InflationLeg, FundShareClass, FlexibleLoan, UnsettledCash, Cash, MasteredInstrument, LoanFacility, FlexibleDeposit, FlexibleRepo") 
    additional_properties: Dict[str, Any] = {}
    __properties = ["instrumentType", "identifiers", "calculationType", "schedules", "originalIssuePrice", "roundingConventions", "assetBacked", "assetPoolIdentifier", "tradingConventions", "timeZoneConventions"]

    @validator('instrument_type')
    def instrument_type_validate_enum(cls, value):
        """Validates the enum"""

        # Finbourne have removed enum validation on all models, except for this use case:
        # Workflow and notification application SDK use the property name 'type' as the discriminator on a number of classes.
        # During instantiation, the value of 'type' is checked against the enum values, 
        

        # check it's a class that uses the 'type' property as a discriminator
        # list of classes can be found by searching for 'actual_instance: Union[' in the generated code
        if 'ComplexBond' not in [ 
                                    # For notification application classes
                                    'AmazonSqsNotificationType',
                                    'AmazonSqsNotificationTypeResponse',
                                    'AmazonSqsPrincipalAuthNotificationType',
                                    'AmazonSqsPrincipalAuthNotificationTypeResponse',
                                    'AzureServiceBusTypeResponse',
                                    'AzureServiceBusNotificationType',
                                    'EmailNotificationType',
                                    'EmailNotificationTypeResponse',
                                    'SmsNotificationType',
                                    'SmsNotificationTypeResponse',
                                    'WebhookNotificationType',
                                    'WebhookNotificationTypeResponse',
                        
                                    # For workflow application classes
                                    'CreateChildTasksAction', 
                                    'RunWorkerAction', 
                                    'TriggerParentTaskAction',
                                    'CreateChildTasksActionResponse', 
                                    'RunWorkerActionResponse',
                                    'TriggerParentTaskActionResponse',
                                    'CreateNewTaskActivity',
                                    'UpdateMatchingTasksActivity',
                                    'CreateNewTaskActivityResponse', 
                                    'UpdateMatchingTasksActivityResponse',
                                    'Fail', 
                                    'GroupReconciliation', 
                                    'HealthCheck', 
                                    'LuminesceView', 
                                    'SchedulerJob', 
                                    'Sleep',
                                    'FailResponse', 
                                    'GroupReconciliationResponse', 
                                    'HealthCheckResponse', 
                                    'LuminesceViewResponse', 
                                    'SchedulerJobResponse', 
                                    'SleepResponse',
                                    'Library',
                                    'LibraryResponse',
                                    'DayRegularity',
                                    'RelativeMonthRegularity',
                                    'SpecificMonthRegularity',
                                    'WeekRegularity',
                                    'YearRegularity']:
           return value
        
        # Only validate the 'type' property of the class
        if "instrument_type" != "type":
            return value

        if value not in ['QuotedSecurity', 'InterestRateSwap', 'FxForward', 'Future', 'ExoticInstrument', 'FxOption', 'CreditDefaultSwap', 'InterestRateSwaption', 'Bond', 'EquityOption', 'FixedLeg', 'FloatingLeg', 'BespokeCashFlowsLeg', 'Unknown', 'TermDeposit', 'ContractForDifference', 'EquitySwap', 'CashPerpetual', 'CapFloor', 'CashSettled', 'CdsIndex', 'Basket', 'FundingLeg', 'FxSwap', 'ForwardRateAgreement', 'SimpleInstrument', 'Repo', 'Equity', 'ExchangeTradedOption', 'ReferenceInstrument', 'ComplexBond', 'InflationLinkedBond', 'InflationSwap', 'SimpleCashFlowLoan', 'TotalReturnSwap', 'InflationLeg', 'FundShareClass', 'FlexibleLoan', 'UnsettledCash', 'Cash', 'MasteredInstrument', 'LoanFacility', 'FlexibleDeposit', 'FlexibleRepo']:
            raise ValueError("must be one of enum values ('QuotedSecurity', 'InterestRateSwap', 'FxForward', 'Future', 'ExoticInstrument', 'FxOption', 'CreditDefaultSwap', 'InterestRateSwaption', 'Bond', 'EquityOption', 'FixedLeg', 'FloatingLeg', 'BespokeCashFlowsLeg', 'Unknown', 'TermDeposit', 'ContractForDifference', 'EquitySwap', 'CashPerpetual', 'CapFloor', 'CashSettled', 'CdsIndex', 'Basket', 'FundingLeg', 'FxSwap', 'ForwardRateAgreement', 'SimpleInstrument', 'Repo', 'Equity', 'ExchangeTradedOption', 'ReferenceInstrument', 'ComplexBond', 'InflationLinkedBond', 'InflationSwap', 'SimpleCashFlowLoan', 'TotalReturnSwap', 'InflationLeg', 'FundShareClass', 'FlexibleLoan', 'UnsettledCash', 'Cash', 'MasteredInstrument', 'LoanFacility', 'FlexibleDeposit', 'FlexibleRepo')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ComplexBond:
        """Create an instance of ComplexBond from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                            "additional_properties"
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in schedules (list)
        _items = []
        if self.schedules:
            for _item in self.schedules:
                if _item:
                    _items.append(_item.to_dict())
            _dict['schedules'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in rounding_conventions (list)
        _items = []
        if self.rounding_conventions:
            for _item in self.rounding_conventions:
                if _item:
                    _items.append(_item.to_dict())
            _dict['roundingConventions'] = _items
        # override the default output from pydantic by calling `to_dict()` of trading_conventions
        if self.trading_conventions:
            _dict['tradingConventions'] = self.trading_conventions.to_dict()
        # override the default output from pydantic by calling `to_dict()` of time_zone_conventions
        if self.time_zone_conventions:
            _dict['timeZoneConventions'] = self.time_zone_conventions.to_dict()
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        # set to None if identifiers (nullable) is None
        # and __fields_set__ contains the field
        if self.identifiers is None and "identifiers" in self.__fields_set__:
            _dict['identifiers'] = None

        # set to None if calculation_type (nullable) is None
        # and __fields_set__ contains the field
        if self.calculation_type is None and "calculation_type" in self.__fields_set__:
            _dict['calculationType'] = None

        # set to None if schedules (nullable) is None
        # and __fields_set__ contains the field
        if self.schedules is None and "schedules" in self.__fields_set__:
            _dict['schedules'] = None

        # set to None if original_issue_price (nullable) is None
        # and __fields_set__ contains the field
        if self.original_issue_price is None and "original_issue_price" in self.__fields_set__:
            _dict['originalIssuePrice'] = None

        # set to None if rounding_conventions (nullable) is None
        # and __fields_set__ contains the field
        if self.rounding_conventions is None and "rounding_conventions" in self.__fields_set__:
            _dict['roundingConventions'] = None

        # set to None if asset_backed (nullable) is None
        # and __fields_set__ contains the field
        if self.asset_backed is None and "asset_backed" in self.__fields_set__:
            _dict['assetBacked'] = None

        # set to None if asset_pool_identifier (nullable) is None
        # and __fields_set__ contains the field
        if self.asset_pool_identifier is None and "asset_pool_identifier" in self.__fields_set__:
            _dict['assetPoolIdentifier'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ComplexBond:
        """Create an instance of ComplexBond from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ComplexBond.parse_obj(obj)

        _obj = ComplexBond.parse_obj({
            "instrument_type": obj.get("instrumentType"),
            "identifiers": obj.get("identifiers"),
            "calculation_type": obj.get("calculationType"),
            "schedules": [Schedule.from_dict(_item) for _item in obj.get("schedules")] if obj.get("schedules") is not None else None,
            "original_issue_price": obj.get("originalIssuePrice"),
            "rounding_conventions": [RoundingConvention.from_dict(_item) for _item in obj.get("roundingConventions")] if obj.get("roundingConventions") is not None else None,
            "asset_backed": obj.get("assetBacked"),
            "asset_pool_identifier": obj.get("assetPoolIdentifier"),
            "trading_conventions": TradingConventions.from_dict(obj.get("tradingConventions")) if obj.get("tradingConventions") is not None else None,
            "time_zone_conventions": TimeZoneConventions.from_dict(obj.get("timeZoneConventions")) if obj.get("timeZoneConventions") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj

ComplexBond.update_forward_refs()



# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Dict, Optional, Any, Union, TYPE_CHECKING
from typing_extensions import Annotated
from pydantic.v1 import BaseModel, StrictStr, StrictInt, StrictBool, StrictFloat, StrictBytes, Field, validator, ValidationError, conlist, constr
from datetime import datetime

class BondConversionEntry(BaseModel):
    """
    Information required to specify a conversion event for a convertible bond.  # noqa: E501
    """
    var_date: Optional[datetime] = Field(default=None, description="The date at which the bond can be converted", alias="date")
    denomination: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of shares to be issued on conversion will be equal to the denomination of the  bond divided by the conversion price.  Two (and only two) entries out of (Price, Ratio, Denomination) must be provided.  So, to allow one entry out of the three to not be provided, we make all the three  nullable defaulting to zero and during validation we check if there is exactly one  of the three equal to zero.")
    price: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The conversion price  Two (and only two) entries out of (Price, Ratio, Denomination) must be provided.  So, to allow one entry out of the three to not be provided, we make all the three  nullable defaulting to zero and during validation we check if there is exactly one  of the three equal to zero.")
    ratio: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of common shares received at the time of conversion for each convertible bond  Two (and only two) entries out of (Price, Ratio, Denomination) must be provided.  So, to allow one entry out of the three to not be provided, we make all the three  nullable defaulting to zero and during validation we check if there is exactly one  of the three equal to zero.")
    __properties = ["date", "denomination", "price", "ratio"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BondConversionEntry:
        """Create an instance of BondConversionEntry from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BondConversionEntry:
        """Create an instance of BondConversionEntry from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BondConversionEntry.parse_obj(obj)

        _obj = BondConversionEntry.parse_obj({
            "var_date": obj.get("date"),
            "denomination": obj.get("denomination"),
            "price": obj.get("price"),
            "ratio": obj.get("ratio")
        })
        return _obj

BondConversionEntry.update_forward_refs()
